#include "tools.h"
#include "CommandLine.h"
#include "toString.h"


namespace ge
{
    CommandLine::CommandLine(int argc, char *argv[])
    {
        fromMain(argc, argv);
    }


    void CommandLine::addMustHaveKeys(const string &keyLine)
    {
        Tokenizer toks(keyLine, ",", false);

        while (toks.hasMoreToken())
            vMustHaveKeys.push_back(toks.nextToken());
    }


    void CommandLine::addOptionalKeys(const string &keyLine)
    {
        Tokenizer toks(keyLine, ",", false);

        while (toks.hasMoreToken())
        {
            vOptionalKeys.push_back(toks.nextToken());
        }
    }


    string CommandLine::getFirstMissingKey() const
    {
        for (unsigned i = 0; i < vMustHaveKeys.size(); i++)
        {
            const string &key = vMustHaveKeys[i];
            if (!hasItem(key))
                return key;
        }


        return "";
    }

    void CommandLine::setArgument(const string &key, const string &arg)
    {
        CommandLineItem cmdItem;

        cmdItem.key = key;
        cmdItem.val = arg;

        mapItems[key] = cmdItem;
    }


    void CommandLine::setChar(const string &key, const char val)
    {
        setArgument(key, toString((int) val));
    }

    void CommandLine::setShort(const string &key, const short val)
    {
        setArgument(key, toString((int) val));
    }

    void CommandLine::setInt(const string &key, const int val)
    {
        setArgument(key, toString((int) val));
    }

    void CommandLine::setLONG(const string &key, const int64_t val)
    {
        setArgument(key, toString(val));
    }

    void CommandLine::setDouble(const string &key, const double val)
    {
        setArgument(key, toString(val));
    }

    void CommandLine::setFloat(const string &key, const float val)
    {
        setArgument(key, toString(val));
    }

    void CommandLine::setBool(const string &key, const bool flag)
    {
        setArgument(key, flag ? "1" : "0");

    }


    int CommandLine::showExample(const string &strExample)
    {
        string s = buildCommandLineExample(true);
        //coutF("%s", s.c_str());
        pp(s);

        if (!hasItem("help"))
            pp("missing --$", getFirstMissingKey().c_str());

        pp("example:");
        pp("$ $", getCommand(), strExample);

        return 1;
    }

    bool CommandLine::checkKeys() const
    {
        for (unsigned i = 0; i < vMustHaveKeys.size(); i++)
        {
            const string &key = vMustHaveKeys[i];


            if (!hasItem(key))
            {
                return false;
            }

        }

        if (hasItem("help"))
            return false;


        return true;
    }

    string CommandLine::buildCommandLineExample(bool flagIncludeOptionalKeys) const
    {
        stringstream ss;

        if (command.length() > 0)
            ss << command << " ";

        for (unsigned i = 0; i < vMustHaveKeys.size(); i++)
        {
            const string &key = vMustHaveKeys[i];
            ss << "--";
            ss << key;
            ss << " arg";

            if (i < vMustHaveKeys.size() - 1)
                ss << " ";
        }

        if (flagIncludeOptionalKeys && vOptionalKeys.size() > 0)
        {
            ss << " [ ";
            for (unsigned i = 0; i < vOptionalKeys.size(); i++)
            {
                const string &key = vOptionalKeys[i];
                ss << "--";
                ss << key;
                ss << " arg";

                if (i < vOptionalKeys.size() - 1)
                    ss << " ";
            }
            ss << " ]";

        }

        return ss.str();
    }


    void CommandLine::fromAdTest()
    {
        //fromMain(AdTest::argc, AdTest::argv);
        //if (AdTest::testMethod)
        //    command = AdTest::testMethod;
    }

    void CommandLine::fromMain(int argc, char *argv[])
    {
        command = "";
        program = "";
        mapItems.clear();

        if (argc < 1)
            return;

        program = argv[0];

        if (argc < 2)
            return;

        command = argv[1];

        if (argc < 3)
            return;

        CommandLineItem cmdItem;


        for (int i = 2; i < argc; i++)
        {
            string s = argv[i];

            if (startsWith(s, "--"))
            {
                s = s.substr(2, s.length());
                cmdItem.key = s;
                mapItems[cmdItem.key] = cmdItem;
                //pp("===$===", s);
            } else
            {

                if (cmdItem.key.length() > 0)
                {
                    CommandLineItem &item = mapItems[cmdItem.key];
                    item.val = s;
                    cmdItem.key = "";
                    cmdItem.val = "";
                }

            }
        }


        /*
    if (cmdItem.key.length() > 0)
    {
        mapItems[cmdItem.key] = cmdItem;
    }
    */

        //if (AdTest::testMethod)
        //  command = AdTest::testMethod;


    }


    bool CommandLine::hasItem(const string &key) const
    {
        typedef map<string, CommandLineItem>::const_iterator ITER;

        ITER it = mapItems.find(getKey(key));
        return it != mapItems.end();
    }

    string CommandLine::getArgument(const string &key) const
    {
        typedef map<string, CommandLineItem>::const_iterator ITER;

        ITER it = mapItems.find(getKey(key));
        return it != mapItems.end() ? it->second.val : "";

    }

    string CommandLine::getCommand() const
    {
        return command;
    }

    string CommandLine::getKey(const string &key) const
    {
        return ocdModuleName.length() > 0 ? ocdModuleName + "." + key : key;
    }


    void CommandLine::show(int space) const
    {
        typedef map<string, CommandLineItem>::const_iterator ITER;


        pp(space, "CommandLine - program: '$'", program);
        pp(space, "CommandLine - command: '$'", command);
        pp(space, "CommandLine - argc:$", mapItems.size());
        for (ITER it = mapItems.begin(); it != mapItems.end(); it++)
        {
            //const string key = getKey(it->first);
            const string key = it->first;
            const CommandLineItem cmdItem = it->second;
            //pp(space+4, "$", key);
            //pp(space+4, ",$=jodel", key);
            //pp(space+4, "$:'$'", key, cmdItem.arg);

            //pp(space+4, ",$=$", cmdItem.key, cmdItem.val);
            pp(space + 4, "$=$", cmdItem.key, cmdItem.val);
        }

    }


    bool CommandLine::isBool(const string &key, bool val) const
    {
        if (!hasItem(key))
            return val;

        const string s = getArgument(key);
        return toBool(s);
    }


    bool CommandLine::getBool(const string &key, bool defaultVal) const
    {
        return isBool(key, defaultVal);
    }


    int CommandLine::getInt(const string &key, int defaultVal) const
    {
        return hasItem(key) ? toInt(getArgument(key)) : defaultVal;

    }

    int64_t CommandLine::getLONG(const string &key, int64_t defaultVal) const
    {
        return hasItem(key) ? toLONG(getArgument(key)) : defaultVal;
    }

    uint64_t CommandLine::getULONG(const string &key, uint64_t defaultVal) const
    {
        return hasItem(key) ? toULONG(getArgument(key)) : defaultVal;
    }

    float CommandLine::getFloat(const string &key, float defaultVal) const
    {
        return hasItem(key) ? toFloat(getArgument(key)) : defaultVal;
    }

    double CommandLine::getDouble(const string &key, double defaultVal) const
    {
        return hasItem(key) ? atof(getArgument(key).c_str()) : defaultVal;
    }

    string CommandLine::getString(const string &key, string defaultVal) const
    {
        return hasItem(key) ? getArgument(key) : defaultVal;
    }


}